"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const string_1 = __importDefault(require("./string"));
class Naming {
    relationAttribute(relation, table) {
        if (relation.columns.length !== 1) {
            return relation.type === 'has-many'
                ? string_1.default.fromSnakeToPluralCamel(relation.on.table)
                : string_1.default.fromSnakeToSingularCamel(relation.on.table);
        }
        if (relation.type !== 'has-many') {
            return string_1.default.fromSnakeToSingularCamel(relation.columns[0].replace(/_id$/, ''));
        }
        //Has many relationship
        const fk = relation.references[0].replace(/_id$/, '');
        if (table === string_1.default.fromSnakeToSingularSnake(fk) ||
            table === string_1.default.fromSnakeToPluralSnake(fk)) {
            return string_1.default.fromSnakeToPluralCamel(relation.on.table);
        }
        return (string_1.default.fromSnakeToPluralCamel(relation.on.table) +
            'As' +
            string_1.default.fromSnakeToSingularPascal(fk));
    }
    capitalizedRelationAttribute(relation, table) {
        return string_1.default.capitalize(this.relationAttribute(relation, table));
    }
    modelClass(table) {
        const name = string_1.default.fromSnakeToSingularPascal(table);
        return name === 'Class' ? name + 'Model' : name;
    }
    controller(table) {
        return string_1.default.fromSnakeToSingularPascal(table) + 'Controller';
    }
    columnAttribute(column) {
        return string_1.default.fromSnakeToCamel(column.name);
    }
    columnLabel(column) {
        return this.label(column.name);
    }
    label(text) {
        return string_1.default.fromSnakeToLabel(text.replace(/_id$/, ''));
    }
    singularLabel(text) {
        return string_1.default.fromSnakeToSingularLabel(text);
    }
    context(table) {
        return string_1.default.fromSnakeToKebab(table);
    }
    relationContext(relation, table) {
        if (relation.columns.length !== 1) {
            return relation.type === 'has-many'
                ? string_1.default.fromSnakeToPluralKebab(relation.on.table)
                : string_1.default.fromSnakeToSingularKebab(relation.on.table);
        }
        if (relation.type !== 'has-many') {
            return string_1.default.fromSnakeToSingularKebab(relation.columns[0].replace(/_id$/, ''));
        }
        //Has many relationship
        const fk = relation.references[0].replace(/_id$/, '');
        if (table === string_1.default.fromSnakeToSingularSnake(fk) ||
            table === string_1.default.fromSnakeToPluralSnake(fk)) {
            return string_1.default.fromSnakeToPluralKebab(relation.on.table);
        }
        return (string_1.default.fromSnakeToPluralKebab(relation.on.table) +
            '-as-' +
            string_1.default.fromSnakeToSingularKebab(fk));
    }
    relationLabel(relation, table) {
        if (relation.columns.length !== 1) {
            return relation.type === 'has-many'
                ? string_1.default.fromSnakeToPluralLabel(relation.on.table)
                : string_1.default.fromSnakeToSingularLabel(relation.on.table);
        }
        if (relation.type !== 'has-many') {
            return string_1.default.fromSnakeToSingularLabel(relation.columns[0].replace(/_id$/, ''));
        }
        //Has many relationship
        const fk = relation.references[0].replace(/_id$/, '');
        if (table === string_1.default.fromSnakeToSingularSnake(fk) ||
            table === string_1.default.fromSnakeToPluralSnake(fk)) {
            return string_1.default.fromSnakeToPluralLabel(relation.on.table);
        }
        return (string_1.default.fromSnakeToPluralLabel(relation.on.table) +
            ' as ' +
            string_1.default.fromSnakeToSingularLabel(fk).toLowerCase());
    }
    listLabel(table) {
        return string_1.default.fromSnakeToPluralLabel(table);
    }
    recordName(table) {
        const recordName = string_1.default.fromSnakeToSingularCamel(table);
        return recordName.length > 32 ? 'record' : recordName;
    }
}
exports.default = new Naming();
