"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pluralize_1 = __importDefault(require("pluralize"));
class StringHelper {
    fromSnakeToSingularPascal(text) {
        return text
            .toLowerCase()
            .split('_')
            .map((part) => {
            return this.capitalize(pluralize_1.default.singular(part));
        })
            .join('');
    }
    fromSnakeToPluralPascal(text) {
        const parts = text.toLowerCase().split('_');
        return parts
            .map((part, index) => {
            return index === parts.length - 1
                ? this.capitalize(pluralize_1.default.plural(part))
                : this.capitalize(pluralize_1.default.singular(part));
        })
            .join('');
    }
    fromSnakeToPascal(text) {
        return text.toLowerCase().split('_').map(this.capitalize).join('');
    }
    fromSnakeToSingularSnake(text) {
        return text.toLowerCase().split('_').map(pluralize_1.default.singular).join('_');
    }
    fromSnakeToPluralSnake(text) {
        const parts = text.toLowerCase().split('_');
        return parts
            .map((part, index) => {
            return index === parts.length - 1
                ? pluralize_1.default.plural(part)
                : pluralize_1.default.singular(part);
        })
            .join('_');
    }
    fromSnakeToSingularKebab(text) {
        return text.toLowerCase().split('_').map(pluralize_1.default.singular).join('-');
    }
    fromSnakeToPluralKebab(text) {
        const parts = text.toLowerCase().split('_');
        return parts
            .map((part, index) => {
            return index === parts.length - 1
                ? pluralize_1.default.plural(part)
                : pluralize_1.default.singular(part);
        })
            .join('-');
    }
    fromSnakeToKebab(text) {
        return text.toLowerCase().split('_').join('-');
    }
    fromSnakeToSingularCamel(text) {
        return this.uncapitalize(this.fromSnakeToSingularPascal(text));
    }
    fromSnakeToPluralCamel(text) {
        return this.uncapitalize(this.fromSnakeToPluralPascal(text));
    }
    fromSnakeToCamel(text) {
        return this.uncapitalize(this.fromSnakeToPascal(text));
    }
    fromSnakeToPluralLabel(text) {
        const parts = text.toLowerCase().split('_');
        return this.capitalize(parts
            .map((part, index) => {
            return index === parts.length - 1
                ? pluralize_1.default.plural(part)
                : pluralize_1.default.singular(part);
        })
            .join(' '));
    }
    fromSnakeToSingularLabel(text) {
        return this.capitalize(text.toLowerCase().split('_').map(pluralize_1.default.singular).join(' '));
    }
    fromSnakeToLabel(text) {
        return this.capitalize(text.toLowerCase().split('_').join(' '));
    }
    //Delete this comment
    //Helpers of helper
    capitalize(text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    }
    uncapitalize(text) {
        return text.charAt(0).toLowerCase() + text.slice(1);
    }
    //Delete this
    className(text) {
        const name = this.fromSnakeToSingularPascal(text);
        return name === 'Class' ? name + 'Model' : name;
    }
    classLabel(text) {
        return this.capitalize(text.toLowerCase().split('_').join(' '));
    }
    columnLabel(text) {
        return this.capitalize(text.toLowerCase().split('_').join(' '));
    }
    toPluralKebab(text) {
        const parts = text.toLowerCase().split('_');
        parts[parts.length - 1] = pluralize_1.default.plural(parts[parts.length - 1]);
        return parts.join('-');
    }
    toPluralLabel(text) {
        const parts = text.toLowerCase().split('_');
        parts[parts.length - 1] = pluralize_1.default.plural(parts[parts.length - 1]);
        return this.capitalize(parts.join(' '));
    }
    toSingularLabel(text) {
        return this.capitalize(text.toLowerCase().split('_').map(pluralize_1.default.singular).join(' '));
    }
    toSingularKebab(text) {
        return text.toLowerCase().split('_').map(pluralize_1.default.singular).join('-');
    }
}
exports.default = new StringHelper();
