"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LaravelGenerator = void 0;
const path_1 = require("path");
const generathor_1 = require("generathor");
const configuration_1 = require("./configuration");
const laravel_1 = require("./transformers/item/laravel");
const menu_1 = require("./transformers/collection/menu");
const route_1 = require("./transformers/collection/route");
const attachRequest_1 = require("./transformers/collection/attachRequest");
const controller_1 = require("./transformers/collection/controller");
const attachForm_1 = require("./transformers/collection/attachForm");
const createRelationForm_1 = require("./transformers/collection/createRelationForm");
const filterRelationForm_1 = require("./transformers/collection/filterRelationForm");
const baseController_1 = require("./transformers/collection/baseController");
const belongsToRelation_1 = require("./transformers/collection/belongsToRelation");
const hasManyRelation_1 = require("./transformers/collection/hasManyRelation");
const handlebars_1 = require("./helpers/handlebars");
class LaravelGenerator {
    constructor(configuration) {
        if (!configuration) {
            configuration = new configuration_1.Configuration({});
        }
        this.$configuration = configuration;
        handlebars_1.HandlebarsHelper.configure();
    }
    templateKey(key) {
        return `${this.$configuration.reference()}-${key}`;
    }
    templateFile(file) {
        return (0, path_1.resolve)(__dirname, `../templates/${file}.handlebars`);
    }
    directory(path) {
        return this.$configuration.directory() + path;
    }
    transformer(item) {
        new laravel_1.Laravel(item, this.$configuration).transform();
    }
    generators() {
        return {
            ...this.generatorsForItems(),
            ...this.generatorsForCollections(),
            ...this.generatorsForStaticFiles(),
        };
    }
    generatorsForItems() {
        const templates = {};
        templates[this.templateKey('parent-model')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('eloquent/parent'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Models/Generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.eloquent),
            fileName: (item) => item.model + '.php',
        });
        if (this.$configuration.createChildModel()) {
            templates[this.templateKey('child-model')] = new generathor_1.GeneratorForItem({
                template: this.templateFile('eloquent/child'),
                source: this.$configuration.source(),
                directory: this.directory('/app/Models'),
                prepareItems: (items) => items.map((item) => item.laravel.eloquent),
                fileName: (item) => item.model + '.php',
            });
        }
        templates[this.templateKey('filter-model')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('eloquent/filter'),
            source: this.$configuration.source(),
            directory: this.directory('/app/ModelFilters'),
            prepareItems: (items) => items.map((item) => item.laravel.filter),
            fileName: (item) => item.class + '.php',
        });
        if (this.$configuration.createEloquentModelsOnly()) {
            return templates;
        }
        templates[this.templateKey('create-request')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/request'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Requests/Generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.requests.create),
            fileName: (item) => item.class + '.php',
        });
        templates[this.templateKey('update-request')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/request'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Requests/Generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.requests.update),
            fileName: (item) => item.class + '.php',
        });
        templates[this.templateKey('filter-request')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/request'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Requests/Generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.requests.filter),
            fileName: (item) => item.class + '.php',
        });
        templates[this.templateKey('attach-request')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/request'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Requests/Generathor'),
            prepareItems: (items) => new attachRequest_1.AttachRequest(items, this.$configuration).transform(),
            fileName: (item) => item.class + '.php',
        });
        templates[this.templateKey('menu')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/menu'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => new menu_1.Menu(items, this.$configuration).transform(),
            fileName: () => 'menu.blade.php',
        });
        templates[this.templateKey('base-controller')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/base-controller'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Controllers/Generathor'),
            prepareItems: (items) => new baseController_1.BaseController(items, this.$configuration).transform(),
            fileName: () => 'Controller.php',
        });
        templates[this.templateKey('create-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.forms.create),
            fileName: (item) => item.context + '/create-form.blade.php',
        });
        templates[this.templateKey('filter-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.forms.filter),
            fileName: (item) => item.context + '/filter-form.blade.php',
        });
        templates[this.templateKey('update-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.forms.update),
            fileName: (item) => item.context + '/update-form.blade.php',
        });
        templates[this.templateKey('attach-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => new attachForm_1.AttachForm(items, this.$configuration).transform(),
            fileName: (item) => `${item.context}/attach-${item.contextFile}-form.blade.php`,
        });
        templates[this.templateKey('create-relation-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => new createRelationForm_1.CreateRelationForm(items, this.$configuration).transform(),
            fileName: (item) => `${item.context}/create-${item.contextFile}-form.blade.php`,
        });
        templates[this.templateKey('filter-relation-form')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/form'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/components/generathor'),
            prepareItems: (items) => new filterRelationForm_1.FilterRelationForm(items, this.$configuration).transform(),
            fileName: (item) => `${item.context}/filter-${item.contextFile}-form.blade.php`,
        });
        templates[this.templateKey('controller')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('others/controller'),
            source: this.$configuration.source(),
            directory: this.directory('/app/Http/Controllers/Generathor'),
            prepareItems: (items) => new controller_1.Controller(items, this.$configuration).transform(),
            fileName: (item) => item.class + '.php',
        });
        templates[this.templateKey('index-view')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/index'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.list),
            fileName: (item) => item.context + '/index.blade.php',
        });
        templates[this.templateKey('edit-view')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/edit'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.edit),
            fileName: (item) => item.context + '/edit.blade.php',
        });
        templates[this.templateKey('show-view')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/show'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => items.map((item) => item.laravel.show),
            fileName: (item) => item.context + '/show.blade.php',
        });
        templates[this.templateKey('relation-item-view')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/relation-item'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => new belongsToRelation_1.BelongsToRelation(items, this.$configuration).transform(),
            fileName: (item) => `${item.context}/${item.relationContext}.blade.php`,
        });
        templates[this.templateKey('relation-list-view')] = new generathor_1.GeneratorForItem({
            template: this.templateFile('views/relation-list'),
            source: this.$configuration.source(),
            directory: this.directory('/resources/views/generathor'),
            prepareItems: (items) => new hasManyRelation_1.HasManyRelation(items, this.$configuration).transform(),
            fileName: (item) => `${item.context}/${item.relationContext}.blade.php`,
        });
        return templates;
    }
    generatorsForCollections() {
        if (this.$configuration.createEloquentModelsOnly()) {
            return {};
        }
        const templates = {};
        templates[this.templateKey('routes')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('others/routes'),
            source: this.$configuration.source(),
            file: this.directory('/routes/generathor.php'),
            prepareItems: (items) => new route_1.Route(items, this.$configuration).transform(),
        });
        return templates;
    }
    generatorsForStaticFiles() {
        const templates = {};
        templates[this.templateKey('pk-trait')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('others/pk-trait'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/app/Models/Generathor/GenerathorKey.php'),
        });
        if (this.$configuration.createEloquentModelsOnly()) {
            return templates;
        }
        templates[this.templateKey('record-input')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('views/record-input'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/resources/views/components/generathor/record-input.blade.php'),
        });
        templates[this.templateKey('breadcrumbs')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('views/breadcrumbs'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/resources/views/components/generathor/breadcrumbs.blade.php'),
        });
        templates[this.templateKey('tabs')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('views/tabs'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/resources/views/components/generathor/tabs.blade.php'),
        });
        templates[this.templateKey('modal')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('views/modal'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/resources/views/components/generathor/modal.blade.php'),
        });
        templates[this.templateKey('loader')] = new generathor_1.GeneratorForCollection({
            template: this.templateFile('views/loader'),
            source: this.$configuration.source(),
            prepareItems: () => [],
            file: this.directory('/resources/views/components/generathor/loader.blade.php'),
        });
        const icons = [
            'check-circle',
            'chevron-right',
            'computer',
            'eye',
            'funnel',
            'home',
            'pencil',
            'plus',
            'trash',
            'x-circle',
            'x',
            'chevron-down',
            'list-bullet',
            'link',
            'unlink',
        ];
        for (const icon of icons) {
            templates[this.templateKey(`icon-${icon}`)] = new generathor_1.GeneratorForCollection({
                template: this.templateFile('views/icons/' + icon),
                source: this.$configuration.source(),
                prepareItems: () => [],
                file: this.directory(`/resources/views/components/generathor/icon-${icon}.blade.php`),
            });
        }
        return templates;
    }
}
exports.LaravelGenerator = LaravelGenerator;
