"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachForm = void 0;
const base_1 = require("./base");
const naming_1 = __importDefault(require("../../helpers/naming"));
class AttachForm extends base_1.BaseTransformer {
    transform() {
        const map = this.mapByTable();
        const result = [];
        for (const item of this.$items) {
            const context = naming_1.default.context(item.table);
            for (const relation of item.relations) {
                if (relation.type !== 'belongs-to') {
                    continue;
                }
                const itemRelated = map[relation.on.table];
                const inputs = [];
                relation.columns.forEach((column) => {
                    inputs.push({
                        type: 'record',
                        name: column,
                        label: naming_1.default.label(column),
                        data: {
                            context: naming_1.default.context(itemRelated.table),
                            relationName: naming_1.default.relationAttribute(relation, itemRelated.table),
                        },
                    });
                });
                const relationLabel = naming_1.default.capitalizedRelationAttribute(relation, itemRelated.table);
                result.push({
                    context,
                    contextFile: naming_1.default.relationContext(relation, item.table),
                    record: null,
                    routeRecord: naming_1.default.recordName(item.table),
                    method: 'post',
                    prefix: '',
                    getOld: 'old',
                    errorBag: 'attach',
                    buttonLabel: 'Save',
                    buttonColor: 'green',
                    columns: '2',
                    route: `generathor.${context}.attach${relationLabel}`,
                    cancelLink: null,
                    inputs: inputs,
                    hiddenInputs: [],
                });
            }
        }
        return result;
    }
}
exports.AttachForm = AttachForm;
