"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BelongsToRelation = void 0;
const base_1 = require("./base");
const naming_1 = __importDefault(require("../../helpers/naming"));
class BelongsToRelation extends base_1.BaseTransformer {
    transform() {
        const result = [];
        const fksByTable = this.fksByTable();
        const map = this.mapByTable();
        for (const item of this.$items) {
            const context = naming_1.default.context(item.table);
            const listLabel = naming_1.default.listLabel(item.table);
            const recordName = naming_1.default.recordName(item.table);
            for (const relation of item.relations) {
                if (relation.type !== 'belongs-to') {
                    continue;
                }
                const relationAttribute = naming_1.default.relationAttribute(relation, item.table);
                const relatedItem = map[relation.on.table];
                const fks = fksByTable[relation.on.table];
                result.push({
                    context,
                    layout: this.$config.layout(),
                    homeRoute: this.$config.homeRoute(),
                    header: listLabel,
                    listLabel: listLabel,
                    relationLabel: naming_1.default.relationLabel(relation, item.table),
                    relationListLabel: naming_1.default.listLabel(relation.on.table),
                    record: recordName,
                    routeRelationRecordName: naming_1.default.recordName(relation.on.table),
                    capitalizedRelationAttribute: naming_1.default.capitalizedRelationAttribute(relation, item.table),
                    relationContext: naming_1.default.relationContext(relation, item.table),
                    routeRelationContext: naming_1.default.context(relation.on.table),
                    relationAttribute: relationAttribute,
                    columns: relatedItem.columns.map((column) => {
                        let type = column.type;
                        let data = null;
                        if (fks[column.name]) {
                            type = 'record';
                            data = fks[column.name];
                        }
                        return {
                            data,
                            type,
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                            nullable: column.nullable,
                        };
                    }),
                    tabs: this.tabs(context, item, recordName, relationAttribute),
                });
            }
        }
        return result;
    }
    tabs(context, item, recordName, relationAttribute) {
        const tabs = [];
        tabs.push({
            label: 'Details',
            route: `generathor.${context}.show`,
            routeRecord: recordName,
        });
        for (const relation of item.relations) {
            const currentRelationAttribute = naming_1.default.relationAttribute(relation, item.table);
            const relationLabel = naming_1.default.relationLabel(relation, item.table);
            if (currentRelationAttribute === relationAttribute) {
                tabs.push({
                    label: relationLabel,
                });
                continue;
            }
            const capitalizedRelationAttribute = naming_1.default.capitalizedRelationAttribute(relation, item.table);
            tabs.push({
                label: relationLabel,
                route: `generathor.${context}.show${capitalizedRelationAttribute}`,
                routeRecord: recordName,
            });
        }
        return tabs;
    }
}
exports.BelongsToRelation = BelongsToRelation;
