"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Controller = void 0;
const base_1 = require("./base");
const naming_1 = __importDefault(require("../../helpers/naming"));
const ignoreAttributesFor = {
    created_at: true,
    updated_at: true,
};
class Controller extends base_1.BaseTransformer {
    transform() {
        const result = [];
        const namespace = 'App\\Http\\Controllers\\Generathor';
        const requestNamespace = 'App\\Http\\Requests\\Generathor';
        const eloquentNamespace = 'App\\Models';
        const itemsByTable = this.mapByTable();
        for (const item of this.$items) {
            const imports = {};
            const hasManyRelations = [];
            const belongsToRelations = [];
            const model = naming_1.default.modelClass(item.table);
            const context = naming_1.default.context(item.table);
            const createRequest = `Create${model}Request`;
            const updateRequest = `Update${model}Request`;
            const filterRequest = `Filter${model}Request`;
            imports[`${eloquentNamespace}\\${model}`] = true;
            imports[`${requestNamespace}\\${createRequest}`] = true;
            imports[`${requestNamespace}\\${updateRequest}`] = true;
            imports[`${requestNamespace}\\${filterRequest}`] = true;
            const controller = naming_1.default.controller(item.table);
            const queryRelations = [];
            for (const relation of item.relations) {
                const label = naming_1.default.relationLabel(relation, item.table);
                queryRelations.push(naming_1.default.relationAttribute(relation, item.table));
                const capitalizedRelationAttribute = naming_1.default.capitalizedRelationAttribute(relation, item.table);
                const relationContext = naming_1.default.relationContext(relation, item.table);
                const relationModel = naming_1.default.modelClass(relation.on.table);
                if (relation.type === 'has-many') {
                    const relationAttribute = naming_1.default.relationAttribute(relation, item.table);
                    const filterRequest = `Filter${relationModel}Request`;
                    imports[`${requestNamespace}\\${filterRequest}`] = true;
                    hasManyRelations.push({
                        label,
                        capitalizedRelationAttribute,
                        relationContext,
                        filterRequest,
                        attribute: relationAttribute,
                        model: relationModel,
                        references: this.references(relation),
                        queryRelations: this.queryRelations(relation, item, itemsByTable),
                    });
                }
                else {
                    const createRelationRequest = `Create${relationModel}Request`;
                    const attachRelationRequest = `Attach${relationModel}To${model}Request`;
                    imports[`${requestNamespace}\\${createRelationRequest}`] = true;
                    imports[`${requestNamespace}\\${attachRelationRequest}`] = true;
                    imports[`${eloquentNamespace}\\${relationModel}`] = true;
                    const references = this.references(relation);
                    const attributes = this.relationAttributes(relation, itemsByTable);
                    belongsToRelations.push({
                        label,
                        attributes,
                        capitalizedRelationAttribute,
                        createRelationRequest,
                        attachRelationRequest,
                        relationContext,
                        model: relationModel,
                        references: references,
                    });
                }
            }
            imports['Illuminate\\Http\\Request'] = true;
            result.push({
                context,
                namespace,
                model,
                createRequest,
                updateRequest,
                filterRequest,
                hasManyRelations,
                belongsToRelations,
                search: this.searchData(item),
                recordName: naming_1.default.recordName(item.table),
                label: naming_1.default.singularLabel(item.table),
                attributes: this.attributes(item),
                class: controller,
                imports: Object.keys(imports).sort(),
                queryRelations: queryRelations.length
                    ? `'${queryRelations.join("', '")}'`
                    : null,
            });
        }
        return result;
    }
    relationAttributes(relation, itemsByTable) {
        const attributes = [];
        const item = itemsByTable[relation.on.table];
        for (const column of item.columns) {
            if (column.autoincrement) {
                continue;
            }
            if (ignoreAttributesFor[column.name]) {
                continue;
            }
            attributes.push({
                name: column.name,
                type: column.type,
            });
        }
        return attributes;
    }
    attributes(item) {
        const attributes = [];
        for (const column of item.columns) {
            if (column.autoincrement) {
                continue;
            }
            if (ignoreAttributesFor[column.name]) {
                continue;
            }
            attributes.push({
                name: column.name,
                type: column.type,
            });
        }
        return attributes;
    }
    searchData(item) {
        if (!item.primaryKey || item.primaryKey.columns.length !== 1) {
            return null;
        }
        const key = item.primaryKey.columns[0];
        for (const column of item.columns) {
            if (column.type === 'string') {
                return {
                    key,
                    attribute: column.name,
                };
            }
        }
        return {
            key,
            attribute: item.columns[0].name,
        };
    }
    queryRelations(currentRelation, item, itemsByTable) {
        const queryRelations = [];
        const relatedItem = itemsByTable[currentRelation.on.table];
        for (const relation of relatedItem.relations) {
            if (relation.on.table === item.table &&
                relation.columns[0] === currentRelation.references[0]) {
                continue;
            }
            queryRelations.push(naming_1.default.relationAttribute(relation, relatedItem.table));
        }
        return queryRelations.length ? `'${queryRelations.join("', '")}'` : null;
    }
}
exports.Controller = Controller;
