"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateRelationForm = void 0;
const base_1 = require("./base");
const naming_1 = __importDefault(require("../../helpers/naming"));
const ignoreInputsFor = {
    created_at: true,
    updated_at: true,
};
class CreateRelationForm extends base_1.BaseTransformer {
    transform() {
        const map = this.mapByTable();
        const result = [];
        const fksMap = this.fksByTable();
        for (const item of this.$items) {
            const context = naming_1.default.context(item.table);
            for (const relation of item.relations) {
                const isHasMayRelation = relation.type === 'has-many';
                const itemRelated = map[relation.on.table];
                const inputs = [];
                const hiddenInputs = [];
                const references = this.references(relation);
                const fks = fksMap[itemRelated.table];
                const recordName = naming_1.default.recordName(item.table);
                itemRelated.columns.forEach((column) => {
                    if (references[column.name]) {
                        if (isHasMayRelation) {
                            hiddenInputs.push({
                                name: column.name,
                                type: 'fixed',
                                data: {
                                    value: `$${recordName}->${references[column.name]}`,
                                },
                            });
                        }
                        return;
                    }
                    if (column.autoincrement) {
                        return;
                    }
                    if (fks[column.name]) {
                        inputs.push({
                            type: 'record',
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                            data: {
                                context: naming_1.default.context(fks[column.name].table),
                                relationName: fks[column.name].relationAttribute,
                            },
                        });
                        return;
                    }
                    if (column.type === 'string' &&
                        column.enum &&
                        Array.isArray(column.enum) &&
                        column.enum.length > 0) {
                        inputs.push({
                            type: 'select',
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                            data: {
                                options: [
                                    {
                                        value: '',
                                        label: 'Select',
                                    },
                                ].concat(column.enum.map(function (value) {
                                    return {
                                        value: value,
                                        label: naming_1.default.label(value),
                                    };
                                })),
                            },
                        });
                        return;
                    }
                    if (column.type === 'int') {
                        inputs.push({
                            type: 'number',
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                            data: {
                                decimals: 0,
                            },
                        });
                        return;
                    }
                    if (column.type === 'float') {
                        inputs.push({
                            type: 'number',
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                            data: {
                                decimals: column.scale,
                            },
                        });
                        return;
                    }
                    if (column.type === 'bool') {
                        inputs.push({
                            type: 'checkbox',
                            name: column.name,
                            label: naming_1.default.columnLabel(column),
                        });
                        return;
                    }
                    const dates = {
                        datetime: true,
                        time: true,
                        date: true,
                    };
                    if (dates[column.subType]) {
                        if (!ignoreInputsFor[column.name]) {
                            inputs.push({
                                type: column.subType,
                                name: column.name,
                                label: naming_1.default.columnLabel(column),
                            });
                        }
                        return;
                    }
                    inputs.push({
                        type: 'text',
                        name: column.name,
                        label: naming_1.default.columnLabel(column),
                    });
                });
                const relationRouteContext = naming_1.default.context(itemRelated.table);
                const parentContext = naming_1.default.context(item.table);
                if (isHasMayRelation) {
                    hiddenInputs.push({
                        name: '_url',
                        type: 'current_url',
                    });
                }
                result.push({
                    context,
                    contextFile: naming_1.default.relationContext(relation, item.table),
                    record: null,
                    routeRecord: isHasMayRelation ? null : recordName,
                    method: 'post',
                    prefix: '',
                    getOld: 'old',
                    errorBag: 'create',
                    buttonLabel: 'Save',
                    buttonColor: 'green',
                    columns: '2',
                    route: isHasMayRelation
                        ? `generathor.${relationRouteContext}.store`
                        : `generathor.${parentContext}.store${naming_1.default.capitalizedRelationAttribute(relation, item.table)}`,
                    cancelLink: null,
                    inputs: inputs,
                    hiddenInputs: hiddenInputs,
                });
            }
        }
        return result;
    }
}
exports.CreateRelationForm = CreateRelationForm;
