"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Route = void 0;
const base_1 = require("./base");
const naming_1 = __importDefault(require("../../helpers/naming"));
class Route extends base_1.BaseTransformer {
    transform() {
        const result = [];
        for (const item of this.$items) {
            const routes = [];
            const context = naming_1.default.context(item.table);
            const controller = this.controller(item.table);
            routes.push({
                context,
                method: 'get',
                path: context,
                action: 'index',
                precognition: true,
            });
            routes.push({
                context,
                method: 'post',
                path: context,
                action: 'store',
                precognition: true,
            });
            if (item.primaryKey && item.primaryKey.columns.length === 1) {
                routes.push({
                    context,
                    method: 'get',
                    path: `${context}/search`,
                    action: 'search',
                    precognition: false,
                });
            }
            const recordName = naming_1.default.recordName(item.table);
            routes.push({
                context,
                method: 'get',
                path: `${context}/{${recordName}}`,
                action: 'show',
                precognition: false,
            });
            routes.push({
                context,
                method: 'get',
                path: `${context}/{${recordName}}/edit`,
                action: 'edit',
                precognition: false,
            });
            routes.push({
                context,
                method: 'put',
                path: `${context}/{${recordName}}`,
                action: 'update',
                precognition: true,
            });
            routes.push({
                context,
                method: 'delete',
                path: `${context}/{${recordName}}`,
                action: 'destroy',
                precognition: false,
            });
            for (const relation of item.relations) {
                const relationContext = naming_1.default.relationContext(relation, item.table);
                const relationName = naming_1.default.capitalizedRelationAttribute(relation, item.table);
                if (relation.type === 'has-many') {
                    routes.push({
                        context,
                        method: 'get',
                        path: `${context}/{${recordName}}/${relationContext}`,
                        action: 'show' + relationName,
                        precognition: true,
                    });
                }
                else {
                    routes.push({
                        context,
                        method: 'get',
                        path: `${context}/{${recordName}}/${relationContext}`,
                        action: 'show' + relationName,
                        precognition: false,
                    });
                    routes.push({
                        context,
                        method: 'post',
                        path: `${context}/{${recordName}}/${relationContext}`,
                        action: 'store' + relationName,
                        precognition: true,
                    });
                    routes.push({
                        context,
                        method: 'post',
                        path: `${context}/{${recordName}}/${relationContext}/attach`,
                        action: 'attach' + relationName,
                        precognition: true,
                    });
                    routes.push({
                        context,
                        method: 'post',
                        path: `${context}/{${recordName}}/${relationContext}/detach`,
                        action: 'detach' + relationName,
                        precognition: true,
                    });
                }
            }
            result.push({
                controller,
                label: naming_1.default.listLabel(item.table),
                routes,
            });
        }
        return result;
    }
    controller(table) {
        return 'App\\Http\\Controllers\\Generathor\\' + naming_1.default.controller(table);
    }
}
exports.Route = Route;
