"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Eloquent = void 0;
const naming_1 = __importDefault(require("../../helpers/naming"));
const type = __importStar(require("../../helpers/type"));
const base_1 = require("./base");
class Eloquent extends base_1.BaseTransformer {
    transform() {
        const primaryKey = this.$item.primaryKey.columns.length === 0
            ? [this.$item.columns[0].name]
            : this.$item.primaryKey.columns;
        const result = {
            model: naming_1.default.modelClass(this.$item.table),
            table: this.$item.table,
            imports: [],
            namespace: 'App\\Models',
            recordLabel: null,
            belongsToRelations: [],
            hasManyRelations: [],
            attributes: [],
            primaryKey: primaryKey.length === 1
                ? `'${primaryKey[0]}'`
                : `['${primaryKey.join("', '")}']`,
            onlyReadAttributes: [],
            autoincrement: false,
            timestamps: false,
            constants: [],
        };
        const imports = {
            [this.$config.eloquentParent(this.$item.table)]: true,
            'Kyslik\\ColumnSortable\\Sortable': true,
            'EloquentFilter\\Filterable': true,
        };
        let importCollection = false;
        this.$item.relations.forEach((relation) => {
            let relationClass = naming_1.default.modelClass(relation.on.table);
            const relationName = naming_1.default.relationAttribute(relation, this.$item.table);
            if (relation.on.table !== this.$item.table) {
                imports['App\\Models\\' + relationClass] = true;
            }
            else {
                imports[`App\\Models\\${relationClass} as ChildModel`] = true;
                relationClass = 'ChildModel';
            }
            let attributeType = relationClass;
            if (relation.type === 'has-many') {
                attributeType = `Collection<${attributeType}>`;
                importCollection = true;
            }
            result.onlyReadAttributes.push({
                name: relationName,
                type: attributeType,
            });
            const columns = {};
            let reference = null;
            if (relation.columns.length === 1) {
                reference =
                    relation.type === 'has-many'
                        ? {
                            foreignKey: relation.references[0],
                            localKey: relation.columns[0],
                        }
                        : {
                            foreignKey: relation.columns[0],
                            ownerKey: relation.references[0],
                        };
            }
            for (const index in relation.columns) {
                columns[relation.references[index]] = relation.columns[index];
            }
            if (relation.type === 'has-many') {
                result.hasManyRelations.push({
                    reference,
                    name: relationName,
                    model: relationClass,
                });
            }
            else {
                result.belongsToRelations.push({
                    reference,
                    name: relationName,
                    model: relationClass,
                });
            }
        });
        let firstStringColumn = null;
        const firstColumn = this.$item.columns[0].name;
        let autoincrement = false;
        const timestamps = {
            created_at: false,
            updated_at: false,
        };
        this.$item.columns.forEach((column) => {
            if (column.enum && Array.isArray(column.enum)) {
                column.enum.forEach((value) => {
                    result.constants.push({
                        name: `${column.name}_${value}`.toUpperCase(),
                        value: value,
                    });
                });
            }
            if (!firstStringColumn && column.type === 'string') {
                firstStringColumn = column.name;
            }
            if (!autoincrement && column.autoincrement) {
                autoincrement = true;
            }
            if (!timestamps.created_at && column.name === 'created_at') {
                timestamps.created_at = true;
            }
            if (!timestamps.updated_at && column.name === 'updated_at') {
                timestamps.updated_at = true;
            }
            if (column.type === 'date') {
                imports['Illuminate\\Support\\Carbon'] = true;
            }
            result.attributes.push({
                type: type.phpType(column.type),
                name: column.name,
            });
        });
        result.recordLabel = firstStringColumn || firstColumn;
        result.autoincrement = autoincrement;
        result.timestamps = timestamps.created_at && timestamps.updated_at;
        if (importCollection) {
            imports['Illuminate\\Database\\Eloquent\\Collection'] = true;
        }
        result.imports = Object.keys(imports).sort();
        this.$item.laravel.eloquent = result;
    }
}
exports.Eloquent = Eloquent;
