"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Forms = void 0;
const naming_1 = __importDefault(require("../../helpers/naming"));
const base_1 = require("./base");
const ignoreInputsFor = {
    created_at: true,
    updated_at: true,
};
class Forms extends base_1.BaseTransformer {
    transform() {
        const context = naming_1.default.context(this.$item.table);
        const recordName = naming_1.default.recordName(this.$item.table);
        const result = {
            create: {
                context,
                record: null,
                routeRecord: null,
                method: 'post',
                prefix: '',
                getOld: 'old',
                errorBag: 'create',
                buttonLabel: 'Save',
                buttonColor: 'green',
                columns: '2',
                route: `generathor.${context}.store`,
                cancelLink: null,
                inputs: [],
                hiddenInputs: [],
            },
            update: {
                context,
                record: recordName,
                routeRecord: recordName,
                method: 'put',
                prefix: '',
                getOld: 'old',
                errorBag: 'update',
                buttonLabel: 'Save',
                buttonColor: 'green',
                columns: '2',
                route: `generathor.${context}.update`,
                cancelLink: {
                    label: 'Cancel',
                    route: `request()->query('_url', route('generathor.${context}.show', ['${recordName}' => $${recordName}->getRouteKey()]))`,
                    icon: null,
                },
                inputs: [],
                hiddenInputs: [
                    {
                        name: '_url',
                        type: 'query',
                    },
                ],
            },
            filter: {
                context,
                record: null,
                routeRecord: null,
                method: 'get',
                prefix: 'filter_',
                getOld: 'request',
                errorBag: 'filter',
                buttonLabel: 'Filter',
                buttonColor: 'blue',
                columns: '4',
                cancelLink: {
                    label: 'Clear filters',
                    route: 'request()->url()',
                    icon: '<x-generathor.icon-x />',
                },
                inputs: [],
                hiddenInputs: [],
            },
        };
        const fks = this.fks();
        this.$item.columns.forEach((column) => {
            if (column.autoincrement) {
                return;
            }
            if (fks[column.name]) {
                const input = {
                    type: 'record',
                    name: column.name,
                    label: naming_1.default.columnLabel(column),
                    data: {
                        context: naming_1.default.context(fks[column.name].table),
                        relationName: fks[column.name].relationAttribute,
                    },
                };
                result.create.inputs.push(input);
                result.update.inputs.push(input);
                result.filter.inputs.push(input);
                return;
            }
            if (column.type === 'string' &&
                column.enum &&
                Array.isArray(column.enum) &&
                column.enum.length > 0) {
                const input = {
                    type: 'select',
                    name: column.name,
                    label: naming_1.default.columnLabel(column),
                    data: {
                        options: [
                            {
                                value: '',
                                label: 'Select',
                            },
                        ].concat(column.enum.map(function (value) {
                            return {
                                value: value,
                                label: naming_1.default.label(value),
                            };
                        })),
                    },
                };
                result.create.inputs.push(input);
                result.update.inputs.push(input);
                result.filter.inputs.push(input);
                return;
            }
            if (column.type === 'int' || column.type === 'float') {
                const decimals = column.type === 'int' ? 0 : column.scale;
                const input = {
                    type: 'number',
                    name: column.name,
                    label: naming_1.default.columnLabel(column),
                    data: {
                        decimals,
                    },
                };
                result.create.inputs.push(input);
                result.update.inputs.push(input);
                result.filter.inputs.push({
                    type: 'number',
                    name: column.name + '_from',
                    label: naming_1.default.label(column.name + '_from'),
                    data: {
                        decimals,
                    },
                });
                result.filter.inputs.push({
                    type: 'number',
                    name: column.name + '_to',
                    label: naming_1.default.label(column.name + '_to'),
                    data: {
                        decimals,
                    },
                });
                return;
            }
            if (column.type === 'bool') {
                const input = {
                    type: 'checkbox',
                    name: column.name,
                    label: naming_1.default.columnLabel(column),
                };
                result.create.inputs.push(input);
                result.update.inputs.push(input);
                result.filter.inputs.push({
                    type: 'select',
                    name: column.name,
                    label: naming_1.default.columnLabel(column),
                    data: {
                        options: [
                            {
                                value: '',
                                label: 'Select',
                            },
                            {
                                value: '0',
                                label: 'No',
                            },
                            {
                                value: '1',
                                label: 'Yes',
                            },
                        ],
                    },
                });
                return;
            }
            const dates = {
                datetime: true,
                time: true,
                date: true,
            };
            if (dates[column.subType]) {
                if (!ignoreInputsFor[column.name]) {
                    const input = {
                        type: column.subType,
                        name: column.name,
                        label: naming_1.default.columnLabel(column),
                    };
                    result.create.inputs.push(input);
                    result.update.inputs.push(input);
                }
                result.filter.inputs.push({
                    type: column.subType,
                    name: column.name + '_from',
                    label: naming_1.default.label(column.name + '_from'),
                });
                result.filter.inputs.push({
                    type: column.subType,
                    name: column.name + '_to',
                    label: naming_1.default.label(column.name + '_to'),
                });
                return;
            }
            const input = {
                type: 'text',
                name: column.name,
                label: naming_1.default.columnLabel(column),
            };
            result.create.inputs.push(input);
            result.update.inputs.push(input);
            result.filter.inputs.push(input);
        });
        this.$item.laravel.forms = result;
    }
}
exports.Forms = Forms;
